/* Copyright (C) 1995, Motorola, Inc. */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <ctype.h>
#include <string.h>


/*
 *   FORMAT string has one or more of following directives:
 *   
 *                white_space -- as identified by "isspace" macro
 *                 characters -- non white space and not '%'
 *       conversion specifier -- '%' followed by character string
 *
 *
 *   ANSI document specifies that each directive is execucted in order
 *   and *scanf routines fail when the first directive fails. Also the
 *   spec. specifies 2 types of failures:
 *
 *       1. input failure     (such a EOF, end of string)
 *       2. matching failure  (bad input)
 *
 *
 *   Strategy:
 *
 *       1. scan in directive from format string
 *       2. execute directive
 *       3. terminate on directive failure, end of input, or end of format
 *
 *
 *   Return Value:
 *
 *       EOF - if input failure occurs before any conversion
 *       Number of input items assigned
 */

int
__doscan( FILE* f, const char* format, va_list ap )
{
    char format_char, stream_char;
    char *bptr, buffer[ 128 ];
    int  items_assigned = 0;
    int  input_chars_scanned = 1;    /* 1 for "preloaded" character */
    int  base, field_width, suppress_assignment;
    long integral_val;
    double float_val;
    enum { NEITHER, SHORT, LONG } short_or_long_specifier = NEITHER;
    enum { NOTHING, DOT, EXP } dot_E = NOTHING;
    enum { IN_SET, NOT_IN_SET } scanset_status = IN_SET;

    /* preload stream_char */
    stream_char = fgetc( f );

    /* scan entire format string */
    while ( '\0' != ( format_char = * format ++ ) )
    {
	/* NON-conversion specifier. no work, just make sure that input
	   matches. */
	if ( '%' != format_char )
	{
	    /* White space */
	    if ( isspace( format_char ) )
	    {
		/* collect characters from the stream, until first NON-white
		   space character. */
		while ( isspace( stream_char ) )
		{
		    stream_char = fgetc( f );
		    input_chars_scanned ++ ;
		}
	    }

	    /* must match the input and format characters */
	    else
	    {
		/* if stream input is EOF, then we have an INPUT error */
		if ( EOF == stream_char )
		{
		    return( ( ! items_assigned ) ? EOF : items_assigned );
		}

		if ( stream_char != format_char )
		{
		    /* matching error has occured, return the
		       non-macthing character to the stream. */
		    ungetc( stream_char, f );

		    return items_assigned;
		}

		/* refill stream character for next time through the loop */
		stream_char = fgetc( f );
		input_chars_scanned ++ ;
	    }
	}
		
	
	/* conversion specifier */
	else
	{
	    /* bump format character past the start of conversion,
	       check for error */
	    if ( '\0' == ( format_char = * format ++ ) )
	    {
		/* format string error */
		return( EOF );
	    }
	    
	    /* just matching */
	    if ( '%' == format_char )
	    {
		/* if stream input is EOF, then we have an INPUT error */
		if ( EOF == stream_char )
		{
		    return( ( ! items_assigned ) ? EOF : items_assigned );
		}

		if ( '%' != stream_char )
		{
		    /* matching error has occured, return the
		       non-macthing character to the stream. */
		    ungetc( stream_char, f );

		    return items_assigned;
		}

		/* refill stream character for next time through the loop */
		stream_char = fgetc( f );
		input_chars_scanned ++ ;
	    }

	    /* actual conversion */
	    else
	    {
		/* check for suppression */
		if ( '*' == format_char )
		{
		    suppress_assignment = 1;

		    /* bump format character past the start of conversion,
		       check for error */
		    if ( '\0' == ( format_char = * format ++ ) )
		    {
			/* format string error */
			return( EOF );
		    }
		}
		else
		{
		    suppress_assignment = 0;
		}

		/* compute the "field width". Note that INT_MAX indicates
		   that field width is not specified. */
		field_width = 0;
		while( isdigit( format_char ) )
		{
		    field_width *= 10;
		    field_width += format_char - '0';

		    /* bump format character past the start of conversion,
		       check for error */
		    if ( '\0' == ( format_char = * format ++ ) )
		    {
			/* format string error */
			return( EOF );
		    }
		}

		if ( ! field_width )
		{
		    field_width = INT_MAX;
		}

		/* check for size of receiving object */
		if ( ( 'l' == format_char ) || ( 'L' == format_char ) )
		{
		    short_or_long_specifier = LONG;

		    /* bump format character past the start of conversion,
		       check for error */
		    if ( '\0' == ( format_char = * format ++ ) )
		    {
			/* format string error */
			return( EOF );
		    }
		}
		
		else if ( 'h' == format_char )
		{
		    short_or_long_specifier = SHORT;

		    /* bump format character past the start of conversion,
		       check for error */
		    if ( '\0' == ( format_char = * format ++ ) )
		    {
			/* format string error */
			return( EOF );
		    }
		}

		/* Input white-space characters are skipped, UNLESS
		   the specification includes a '[', 'c', or 'n' */
		if ( ( format_char != '[' ) &&
		     ( format_char != 'c' ) &&
		     ( format_char != 'n' ) )
		{
		    while ( isspace( stream_char ) )
		    {
			/* get next stream character */
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
		    }
		}
		
		/* Leave a NEW character in stream_char when DONE with CASE */
		switch ( format_char )
		{
		    /* number of input chars read so far */
		case 'n':
		    if ( ! suppress_assignment )
		    {
			/* REMEMBER: we are 1 stream character ahead */
			if ( LONG != short_or_long_specifier )
			{
			    *va_arg( ap, int * ) = input_chars_scanned - 1;
			}
			else
			{
			    *va_arg(ap, long *) = (long)input_chars_scanned -1;
			}
		    }
		    break;

		    /* pointer sequence, as in printf */
		case 'p':
		    /* optionally signed decimal integer */
		case 'd':
		    /* optionally signed integer */
		case 'i':
		    /* optionally signed octal integer */
		case 'o':
		    /* optionally signed decimal integer */
		case 'u':
		    /* optionally signed hexadecimal integer */
		case 'x':
		case 'X':
		    if ( ( 'x' == format_char ) || ( 'X' == format_char ) )
		    {
			base = 16;
		    }
		    else if ( 'o' == format_char )
		    {
			base = 8;
		    }
		    else if ( 'i' == format_char )
		    {
			base = 0;
		    }
		    else if ( 'p' == format_char )
		    {
			base = 16;
#if defined( __DSP56K__ ) || defined( __DSP563C__ ) || defined( __DSP566C__ )
			field_width = 6;
#else
			field_width = 10;
#endif
		    }
		    else
		    {
			base = 10;
		    }
		    
		    /* initialize ptr to beginning of number buffer */
		    bptr = buffer;

		    /* check for leading sign */
		    if ( ( '+' == stream_char ) || ( '-' == stream_char ) )
		    {
			/* jam the sign into the number buffer */
			*bptr ++ = stream_char;

			/* adjust field_width, check for field_width
			   error later down stream */
			if ( 0 < field_width )
			{
			    field_width -- ;
			}

			/* get next stream character */
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
		    }
		    
		    /* leading '0' (octal) or "0x" (hex)?, no need to
		       pass to number buffer */
		    if ( '0' == stream_char )
		    {
			/* adjust field_width, check for field_width
			   error later down stream */
			if ( 0 < field_width )
			{
			    field_width -- ;
			}

                        *bptr++ = '0'; /* MSIL */
			/* get next stream character */
			stream_char = fgetc( f );
			input_chars_scanned ++ ;

			/* hex */
			if ( ( 'x' == stream_char ) || ( 'X' == stream_char ) )
			{
			    if ( 0 == base )
			    {
				base = 16;
			    }

			    /* check to see if collecting hex value */
			    if ( 16 == base )
			    {
				if ( 0 < field_width )
				{
				    field_width -- ;
				}

				stream_char = fgetc( f );
				input_chars_scanned ++ ;
			    }
			}
			else if ( 0 == base )
                        {
                            base = 8;
                        }
		    }

		    /* by this time, if base is 0 -- force to 10 */
		    if ( 0 == base )
		    {
			base = 10;
		    }
		    
		    /* collect number */
		    while ( ( EOF != stream_char ) && ( 0 < field_width ) )
		    {
			if ( ( 16 == base ) )
			{
			    if ( 0 == isxdigit( stream_char ) )
			    {
				break;
			    }
			}
			else if ( 0 == isdigit( stream_char ) ||
				    ( ( stream_char - '0' ) > base ) )
			{
			    break;
			}
			
			*bptr ++ = stream_char;

			stream_char = fgetc( f );
			input_chars_scanned ++ ;
			field_width -- ;
		    }

		    /* check for at least 1 digit */
		    if ( ! isxdigit( *(bptr - 1) ) )
		    {
			/* matching error has occured, return the
			   non-macthing character to the stream. */
			ungetc( stream_char, f );

			return items_assigned;
		    }

		    /* number seems ok, so terminate it */
		    *bptr = '\0';

		    /* convert and count conversion, when necessary */
		    if ( ! suppress_assignment )
		    {
			/* convert number string */
			if ( ( 'd' == format_char ) || ( 'i' == format_char ) )
			{
			    integral_val = strtol( buffer, & bptr, base );
			}
			else
			{
			    integral_val = strtoul( buffer, & bptr, base );
			}
		    
			if ( LONG != short_or_long_specifier )
			{
			    *va_arg( ap, int * ) = (int) integral_val;
			}
			else
			{
			    *va_arg( ap, long * ) = integral_val;
			}

			items_assigned ++ ;
		    }
		    break;

		    /* optionally signed floating-point */
		case 'e':  case 'E':
		case 'f':
		case 'g':  case 'G':
		    /* initialize ptr to beginning of the number buffer */
		    bptr = buffer;

		    /* check for leading sign */
		    if ( ( '+' == stream_char ) || ( '-' == stream_char ) )
		    {
			/* jam the sign into the number buffer */
			*bptr ++ = stream_char;

			/* adjust field_width, check for field_width
			   error later down stream */
			if ( 0 < field_width )
			{
			    field_width -- ;
			}

			/* get next stream character */
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
		    }

		    /* collect number */
		    while ( ( EOF != stream_char ) && ( 0 < field_width ) )
		    {
			if ( isdigit( stream_char ) )
			{
			    *bptr ++ = stream_char;
			}

			/* decimal point */
			else if ( ( '.' == stream_char ) &&
				 ( ! ( DOT & dot_E ) ) )
			{
			    dot_E |= DOT;
			    *bptr ++ = stream_char;
			}
			
			/* exponent part 'e' or 'E' */
			else if ( ( ( 'e' == stream_char ) ||
				    ( 'E' == stream_char ) ) &&
				 ( ! ( EXP & dot_E ) ) )
			{
			    dot_E |= ( EXP | DOT );
			    *bptr ++ = tolower( stream_char );
			}
			
			/* possible optional sign */
			else if ( ( ( '+' == stream_char ) ||
                                    ( '-' == stream_char ) ) &&
				 ( 'e' == *(bptr - 1) ) )
			{
			    *bptr ++ = tolower( stream_char );
			}
			
			else
			{
			    /* floating suffix */
			    if ( ( 'f' == stream_char ) ||
				 ( 'F' == stream_char ) ||
				 ( 'l' == stream_char ) ||
				 ( 'L' == stream_char ) )
			    {
				stream_char = fgetc( f );
				input_chars_scanned ++ ;
			    }
				
			    break;
			}
			
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
			field_width -- ;
		    }
		    
		    /* check for floating point format errors */
		    if ( '+' == *(bptr - 1) || '-' == *(bptr - 1) ||
			 'e' == *(bptr - 1) || bptr == buffer )
		    {
			/* matching error has occured, return the
			   non-macthing character to the stream. */
			ungetc( stream_char, f );

			return items_assigned;
		    }			

		    /* no format errors -- so terminate string */
		    *bptr = '\0';

		    if ( ! suppress_assignment )
		    {
			float_val = strtod( buffer, &bptr );

			if ( LONG != short_or_long_specifier )
			{
			    *va_arg( ap, float * ) = (float) float_val;
			}
			else
			{
			    *va_arg( ap, double * ) = float_val;
			}

			items_assigned ++ ;
		    }
		    break;

		    /* sequence of non-white space characters */
		case 's':
		{
		    char* ptr;
		    
		    /* "ptr" contains address of buffer large enough
		       to hold field_width characters */
		    ptr = va_arg( ap, char * );

		    while ( ( EOF != stream_char ) && ( 0 < field_width-- ) )
		    {
			/* most cases should break out here */
			if ( isspace( stream_char ) )
			{
			    break;
			}

			if ( ! suppress_assignment )
			{
			    *ptr++ = stream_char;
			}

			/* reload */
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
		    }

		    if ( ! suppress_assignment )
		    {
			/* terminate the string */
			*ptr = '\0';
			items_assigned ++ ;
		    }
		}
		    break;

		    /* nonempty sequence of chars from a scanset */
		case '[':
		{
		    char* ptr;

		    /* "ptr" contains address of buffer large enough
		       to hold field_width characters */
		    ptr = va_arg( ap, char * );

		    /* bump into scanset */
		    format_char = *format ++ ;

		    /* initialize ptr to beginning of buffer for scanset */
		    bptr = buffer;

		    if ( '^' == format_char )
		    {
			format_char = *format ++ ;

			/* add hard right bracket to scanset */
			if ( ']' == format_char )
			{
			    *bptr ++ = ']';
			    format_char = *format ++ ;
			}

			scanset_status = NOT_IN_SET;
		    }

		    /* add hard right bracket to scanset */
		    else if ( ']' == format_char )
		    {
			*bptr ++ = ']';
			format_char = *format ++ ;
		    }

		    /* collect the scanset */
		    while ( ( '\0' != format_char ) && ( ']' != format_char ) )
		    {
			*bptr ++ = format_char;
			format_char = *format ++ ;
		    }
		    
		    /* dust ']' */
		    if ( ']' == format_char )
		    {
			format_char = *format ++ ;
		    }

		    while ( ( EOF != stream_char ) && ( 0 < field_width -- ) )
		    {
			if ( ( NULL == strchr( buffer, stream_char ) ) != scanset_status )
			{
			    break;
			}
			
			if ( ! suppress_assignment )
			{
			    *ptr ++ = stream_char;
			}
			
			stream_char = fgetc( f );
			input_chars_scanned ++ ;
		    }

		    if ( ! suppress_assignment )
		    {
			*ptr = '\0';
			items_assigned ++ ;
		    }
		}
		    
		    break;

		    /* sequence of chars of the number specified by
		       the field-width (minimum 1 ) */
		case 'c':
		{
		    char* ptr;
		    
		    /* "ptr" contains address of buffer large enough
		       to hold field_width characters */
		    ptr = va_arg( ap, char * );

		    /* no field width specified -- default to 1 */
		    if ( field_width == INT_MAX )
		    {
			field_width = 1;
		    }

		    if ( ! suppress_assignment )
		    {
			while ( (EOF != stream_char) && (0 < field_width--) )
			{
			    *ptr++ = stream_char;

			    /* reload */
			    stream_char = fgetc( f );
			    input_chars_scanned ++ ;
			}
			
			items_assigned ++ ;
		    }
		    else
		    {
			while ( (EOF != stream_char) && (0 < field_width--) )
			{
			    /* reload */
			    stream_char = fgetc( f );
			    input_chars_scanned ++ ;
			}
		    }
		}
		    break;
		}
	    }
	}
    }

    /* by reaching this point, no errors occured above AND the character
       "stream_char" must be shoved back into the stream. DONT shove the
       EOF character back into stream. */
    if ( EOF != stream_char )
    {
	ungetc( stream_char, f );
    }

    return( items_assigned );
}
