/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

/* forward function declarations */
static FILE* __do_open( const char*, const char*, FILE* );

/* externally declared global variables */
extern FILE* __open_files[];

/*
 * FOPEN():
 *    The goal here is to open the file "filename" with the correct modes.
 *
 * Note:
 *    1. opening a file with 'r' as first character in mode argument fails if
 *       the file does not exist or can not be read.
 *    2. opening file with append mode ('a') causes all subsequent writes to 
 *       the file to be forced to the current EOF, regardless of intervening 
 *       calls to the fseek function.
 *    3. when opened, a stream is FULLY buffered and the error and EOF
 *       indicators are cleared.
 *
 *    function return value:
 *
 *    ERRNO:
 *        unused.
 */

FILE *
fopen( const char* filename, const char* mode )
{
    FILE* stream;
    int next_stream = 0;

    /* find the next ID available */
    while ( __open_files[ next_stream ] != NULL )
    {
	next_stream ++ ;

	/* check for error */
	if ( next_stream > ( FOPEN_MAX + 3 ) )
	{
	    errno = ENOFILEDESC;
	    return NULL;
	}
    }
    /* "register" the files stream */
    stream = __open_files[ next_stream ] = & __streams[ next_stream ];

    /* actually do the file opening */
    stream = __do_open( filename, mode, stream );

    return stream;
}



/*
 * FREOPEN():
 *    The goal here is to open 
 */
FILE *
freopen ( const char* filename, const char* mode, FILE* stream )
{
    int i;
    
    /* find the stream index. */
    for ( i = 0; i < FOPEN_MAX + 3; ++ i )
    {
	if ( stream == __open_files[i] )
	{
	    break;
	}
    }
    
    /* From ANSI document --
       "Failure to close the file successfully is ignored" */
    (void) fclose( stream );

    stream = __open_files[i] = __do_open( filename, mode, stream );
    
    return stream;
}



/*
 *
 */
static FILE*
__do_open( const char* filename, const char* mode, FILE* stream )
{
    char* tmp;
    int   is_binary = 0, open_flag = 0, update = 0;
    
    /* clear stream _flag value */
    stream->_flag = 0;

    /* determine the "flag", "stock" mode string can be up 3 chars long */
    
    /* check for "update" or "binary" opening */
    tmp = (char*) mode + 1;
    while ( *tmp != '\0' )
    {
	/* update */
	if ( *tmp == '+' )
	    update = 1;
	
	/* binary */
	if ( *tmp == 'b' )
	{
	    is_binary = 1;
	    stream->_flag = _IOBIN;
	}

	tmp ++ ;
    }
    
    switch ( mode[0] )
    {
    case 'w':
	if ( update )
	{
	    stream->_flag |= _IORW;
	    open_flag = (__O_RDWR | __O_TRUNC | __O_CREAT);
	}
	else
	{
	    stream->_flag |= _IOWRITE;
	    open_flag = (__O_WRONLY | __O_TRUNC | __O_CREAT);
	}

	break;
	
    case 'a':
	if ( update )
	{
	    stream->_flag |= _IORW;
	    open_flag = (__O_RDWR | __O_APPEND | __O_CREAT);
	}
	else
	{
	    stream->_flag |= _IOWRITE;
	    open_flag = (__O_WRONLY | __O_APPEND | __O_CREAT);
	}

	break;
	
    case 'r':
	if ( update )
	{
	    stream->_flag |= _IORW;
	    open_flag = __O_RDWR;
	}
	else
	{
	    stream->_flag |= _IOREAD;
	    open_flag = __O_RDONLY;
	}
	
	break;
	
    default:
	/* error case -- bad mode string */
	return NULL;
    }

    if ( is_binary )
    {
	open_flag |= __O_BINARY;
    }

    /* return value is the negative errno from the host */
    if ( ( stream->_file = __open( filename, open_flag ) ) < 0 )
    {
	/* file open error */
	return NULL;
    }

    /* if the file was opened for "append", then lseek to the end */
    if ( open_flag & __O_APPEND )
    {
	long offset;
	
	if ( ( offset = __lseek( stream->_file, 0l, __L_XTND ) ) < 0 )
	{
	    /* seeking error has occured, offset == -errno */
	    return NULL;
	}
    }

    /* open all files with BUFSIZ buffers. setbuf and setvbuf can reset */
    stream->_bufsiz = BUFSIZ;
    
    /* after all error posibilities have been passed over, allocate a buffer */
    if ( NULL == ( stream->_buffer = malloc( BUFSIZ + 1 ) ) )
    {
	/* failed to allocate a buffer */
	return NULL;
    }

    /* flip off __O_BINARY - because __O_RDONLY can be 0 */
    if ( is_binary )
    {
	open_flag &= ~__O_BINARY;
    }

    /* initialize remaining stream elements */
    if ( open_flag == __O_RDONLY )
    {
	stream->_cnt = 0;
    }
    else
    {
	stream->_cnt = stream->_bufsiz;
    }

    stream->_flag |= ( _IOFBF | _IOABUF ) ;

    /* make room for the 1 extra character that MAY be ungetc'd before
       stream filled */
    stream->_buffer ++ ;

    stream->_ptr = stream->_buffer;

    /* return FILE* */
    return stream;
}
