/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>
#include <stdlib.h>

/*
 * FWRITE():
 *    The goal here is to write up NMEMB data elements of size SIZE
 *    that are pointed to by PTR to the stream pointed to by STREAM.
 *
 *    Error situations to be aware of:
 *      1. PTR is NULL    -- No data to write
 *      2. SIZE <= 0      -- invalid data size
 *      3. NMEMB is <= 0  -- Again, no data to write
 *      4. STREAM is not valid  -- non-write stream
 *
 *    Return value:
 *      The number of data elements written.
 *      NOTE: an error has occured if the return value does NOT equal NMEMB.
 *      According to the ANSI spec, there is no ERRNO usage upon errro.
 *
 *    ERRNO:
 *      unused.
 */

size_t
fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream )
{
    size_t chunks;
    extern int __already_registered_p;
    void __stdio_cleanup ( );
    
    /* ensure that all buffers will eventually be flushed. */
    if ( ! __already_registered_p )
    {
	atexit ( __stdio_cleanup );
	__already_registered_p = 1;
    }

    /* check for basic errors */
    if ( NULL == ptr || 0 >= size || 0 >= nmemb ||
	 ( !( stream->_flag & _IOWRITE ) && !( stream->_flag & _IORW ) ) )
    {
	stream->_flag |= _IOERR;
	return (size_t) 0;
    }
    
    /* check for EOF error */
    if ( stream->_flag & _IOEOF )
    {
	/* input cool on update buffers */
	if ( ! ( stream->_flag & _IORW ) )
	{
	    stream->_flag |= _IOERR;
	    return (size_t) 0;
	}
	else
	{
	    stream->_flag &= ~_IOEOF;
	}
    }

    /* calculate the total number of "chunks" to write. On the DSP56000/1/2,
       DSP96002, and the DSP56156 there is no support for "byte" operations.
       Therefore, __write operates on the machine's native data word.

       for example, on the DSP56000/1/2 (in x or y mode)
           sizeof(char) == 1
            sizeof(int) == 1
           sizeof(long) == 2
    */
    chunks = size * nmemb;
    
    /* if the stream is UNBUFFERED (_IONBF)  - or -
       the current buffer is empty AND the number of chunks to write is greater
       than or equal to the buffer's size (BUFSIZ) it is OK to directly
       write the buffer.
    */
    if ( stream->_flag & _IONBF ||
	(( stream->_buffer == stream->_ptr ) &&
	 ( chunks >= stream->_bufsiz )))
    {
	size_t written;
	
	if (( written = __write( stream->_file, ptr, chunks )) != chunks )
	{
	    stream->_flag |= _IOERR;
	}

	/* reset the buffer */
	stream->_cnt = stream->_bufsiz;

	/* clear errors */
	stream->_flag &= ~(_IOEOF | _IOERR);

	/* faster to assume "update" buffer than to check */
	stream->_flag &= ~_IOREAD;
	stream->_flag |= _IOWRITE;

	stream->_ptr = stream->_buffer;
	
	return (size_t) ( written / size );
    }

    /* BUFFERED output */

    /* if update stream was NOT last used as a _IOWRITE stream, assume that
       the programmer performed the file positioning as REQUIRED by ANSI. */
    if ( !( stream->_flag & _IOWRITE ) )
    {
	stream->_flag &= ~_IOREAD;
	stream->_flag |= _IOWRITE;
	stream->_cnt = stream->_bufsiz;   /* the entire buffer is available */
    }

    /* if the size of the new data to be written (chunks) is less than the
       remaining buffer, just throw it into the buffer.    */
    if ( chunks < stream->_cnt )
    {
	void *memcpy(), *memchr();

	/* copy into buffer, regardless of buffering */
	memcpy( stream->_ptr, ptr, chunks );

	/* update stream to reflect new data added to the buffer */
	stream->_cnt -= chunks;    /* less available space */
	stream->_ptr += chunks;    /* next available spot in buffer */

	/* must check for '\n' in line buffered streams and flush at each.
	   it is faster to jam the entire addition into the waiting buffer and
	   then check, then to make a bunch of calls to fputc.  */
	if ( stream->_flag & _IOLBF )
	{
	    int  written, data_written = 0;
	    char *head, *tail;
	    
	    /* finding embedded newline characters */
	    head = stream->_buffer;
	    tail = stream->_ptr - chunks;
	    
	    /* while buffer has data */
	    while ( tail != stream->_ptr )
	    {
		if ( *tail ++ == '\n' )
		{
		    /* write buffer */
		    data_written += written = tail - head;
		    if ( __write(stream->_file, head, written) != written )
		    {
			/* error has occured */
			stream->_flag |= _IOERR;
			return (size_t) 0;
		    }
		    
		    head = tail;
		}
	    }

	    /* if stream has been "line flushed" -- adjust stream info */
	    if ( data_written )
	    {
		int new_size;
		
		/* space has been made available in buffer */
		stream->_cnt += data_written;

		/* shift the buffer over */
		new_size = stream->_bufsiz - stream->_cnt;
		memcpy( stream->_buffer, head, new_size );

		/* adjust _ptr to the next available */
		stream->_ptr = stream->_buffer + new_size;

		/* clear errors */
		stream->_flag &= ~(_IOEOF | _IOERR);
	    }
	}
	
	return nmemb;
    }
    
    /* the new data fills the buffer, so write out buffer and then write
       out all of the new data.
    */ 
    {
	size_t written;
	int to_write = stream->_bufsiz - stream->_cnt;

	if ( __write( stream->_file, stream->_buffer, to_write ) != to_write )
	{
	    stream->_flag |= _IOERR;

	    /* return 0 because the buffer size may be the same
	       size as new data */
	    return (size_t) 0;
	}

	if ( ( written = __write( stream->_file, ptr, chunks ) ) != chunks )
	{
	    stream->_flag |= _IOERR;
	}

	/* clean up stream after successful write */
	stream->_cnt = stream->_bufsiz;
	stream->_flag &= ~(_IOEOF | _IOERR);
	stream->_ptr = stream->_buffer;

	return (size_t) ( written / size );
    }
}
