/* Copyright (C) 1995, by Motorola, Inc. */

/*
 *	pow(x, y) returns x ** y.
 *	Returns EDOM error and value 0 for 0 to a non-positive power
 *	or negative to a non-integral power;
 *	ERANGE error and value HUGE or -HUGE when the correct value
 *	would overflow, or 0 when the correct value would underflow.
 *	uses log and exp
 *	This version accepts negative x and integral y,
 *	apparently in violation of the ANSI FORTRAN standard for x ** y.
 *	There is a much more accurate but much more elaborate algorithm
 *	in Cody and Waite, which should be substituted for this.
 */

#include <float.h>
#include <math.h>
#include "trans.h"
#include <errno.h>

double
pow( double x, double y)
{
    int neg;
    double dummy;

    /* checking for domain errors. */
    if ((( 0.0 > x ) && ( 0.0 != modf ( y, & dummy ))) ||
	(( 0.0 == x ) && ( 0.0 >= y )))
    {
	errno = EDOM;
	
	return 0.0;
    }
    
    /* easy special cases */
    if ( 0 == y )
    {
	return( 1.0 );
    }
    
    else if ( 1 == y )
    {
	return (x);
    }


    if (  0 == x )
    {
	return( 0.0 );
    }
    
    neg = 0;

    /* test using integer arithmetic if possible */
    if ( x < 0 )
    {
	if ( y >= -MAXLONG && y <= MAXLONG )
	{
	    long ly = (long)y;
	    
	    if ((double)ly != y)
	    {
		goto domain; /* y not integral */
	    }

	    neg = ly % 2;
	}

	else
	{
	    double fr, dum;
	    
	    if ( fr = modf(0.5 * y, &dum))
	    {
		if ( fr != 0.5 )
		{
		    goto domain; /* y not integral */
		}

		neg++; /* y is odd */
	    }
	}

	x = -x;
    }

    
    if ( neg < 0 )
    {
	neg = -neg;
    }
    

    /* x isn't the final result */
    if ( x != 1 )
    {
	/* the following code protects against multiplying x and y
	 * until there is no chance of multiplicative overflow */

	/* preserve sign of product */
	if ( (x = log(x)) < 0 )
	{
	    x = -x;
	    y = -y;
	}

	if ( y > LN_MAXDOUBLE/x )
	{
	    errno = ERANGE;
	    
	    return ( ( neg == 1 ) ? -HUGE : HUGE );
	}

	if ( y < LN_MINDOUBLE/x )
	{
	    errno = ERANGE;
	    
	    return ( 0.0 );
	}

	x = exp(x * y); /* finally; no mishap can occur */
    }

    return ( ( neg == 1 ) ? -x : x);

domain:
    errno = ERANGE;

    return ( 0.0 );
}
