/* Copyright (C) 1995, by Motorola, Inc. */

/*
 *	tan returns the tangent of its double-precision argument.
 *	Returns ERANGE error and value 0 if argument too large.
 *	Algorithm and coefficients from Cody and Waite (1980).
 */

#include <math.h>
#include "trans.h"
#include <errno.h>


#define _REDUCE(TYPE, X, XN, C1, C2)    { \
	double x1 = (double)(TYPE)X, x2 = X - x1; \
	X = x1 - (XN) * (C1); X += x2; X -= (XN) * (C2); }

double
tan( double x)
{
	double y;
	int neg = 0, invert;

	if ( x < 0 )
	{
		x = -x;
		neg++;
	}

	if ( x > X_TLOSS/2 )
	{
		return ( 0.0 );
	}

	y = x * M_2_PI + 0.5;

	/* reduce using integer arithmetic if possible */
	if ( x <= MAXLONG )
	{
		long n = (long)y;

		y = (double)n;
		_REDUCE(long, x, y, 1.57080078125, -4.454455103380768678308e-6);
		invert = (int)n % 2;
	}
	else
	{
		double dn;

		x = (modf(y, &dn) - 0.5) * M_PI_2;
		invert = modf(0.5 * dn, &dn) ? 1 : 0;
	}

	if ( x > -X_EPS && x < X_EPS )
	{
			/*
			 * This case represents a range reduction
			 * from (2n + 1) * PI/2 which happens to return 0.
			 * This is mathematically insignficant, and extremely
			 * unlikely in any case, so it is best simply
			 * to substitute an arbitrarily small value
			 * as if the reduction were one bit away from 0,
			 * and not to deal with this as a range error.
			 */
		if ( !x && invert )
		{
			x = 1.0/X_TLOSS;
		}

		y = 1.0; /* skip for efficiency and to prevent underflow */
	}
	else
	{
		static double p[] = {
			-0.17861707342254426711e-4,
			 0.34248878235890589960e-2,
			-0.13338350006421960681e+0,
		}, q[] = {
			 0.49819433993786512270e-6,
			-0.31181531907010027307e-3,
			 0.25663832289440112864e-1,
			-0.46671683339755294240e+0,
			 1.0,
		};

		y = x * x;
		x += x * y * _POLY2(y, p);
		y = _POLY4(y, q);
	}


	if ( neg )
	{
		x = -x;
	}

	return( invert ? -y/x : x/y );
}
