/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/cfthru/main.c                       $
 *     $Date:: 14/05/02 17:12                                                  $
 * $Revision:: 4                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/******************************************************************************
 This example shows how to get audio data from the DSP to the ColdFire and back
 again. It only uses the volume potentiometer to adjust the desired volume
 using a pre-filled table with a variable gain from -90 dB to 0 dB.
 The volume is updated using a timer procedure to interpolate intermediate 
 points allowing smoother changes than using directly the valued got from the
 panel when it is received.
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <rtems.h>
#include <chameleon.h>

#include "dsp/dsp_code.h"

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

static rtems_signed16 master_volume_target = 0;
static rtems_signed16 master_volume = 0;

void Error(char *error)
{
	TRACE(error);
	exit(1);
}

rtems_timer_service_routine timer_routine(rtems_id timer_id, void *user_data)
{
	rtems_signed32 diff = master_volume - master_volume_target;
	
	if (diff != 0)
	{
		diff >>= 5;
		master_volume -= diff;
	}
	
	rtems_timer_reset(timer_id);
}

static rtems_task panel_task(rtems_task_argument argument)
{
	static rtems_signed32	volume_table[128];
	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	rtems_status_code	status;
	rtems_id		timer_id;
	int			panel;
	int			i;
	float			dB;
	char			text[17];

	TRACE("Starting panel_task...\n");

	status = rtems_timer_create(rtems_build_name('V', 'O', 'L', 'T'), &timer_id);
	if (status != RTEMS_SUCCESSFUL)
		Error("Can't create the volume timer.\n");
		
	status = rtems_timer_fire_after(timer_id, 2, timer_routine, 0);
	if (status != RTEMS_SUCCESSFUL)
		Error("Can't start the volume timer.\n");
		
	/* Fill volume_table with a variable gain from -90 dB to 0 dB */
	for (i=0; i<128 ;i++)
	{
		if (i < 27)
			dB = -90.0 + (float) 40.0*i/27.0;
		else
			dB = -50.0 + (float) 50.0*(i-27)/100.0;

		volume_table[i] = (rtems_signed16) (float_to_fix_round(pow(10.0, dB/20.0)) >> 8);
	}

	volume_table[0] = 0;
	
	panel = panel_init();
	if (!panel)
		Error("ERROR: cannot access the Panel.\n");

	panel_out_lcd_print(panel, 0, 0, " ColdFire thru  ");

	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
			Error("ERROR: unexpected exit waiting new panel event.\n");

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			switch (potentiometer)
			{
				case PANEL01_POT_VOLUME:
					master_volume_target = volume_table[value];
					sprintf(text, "   Volume %-3d   ", value);
					panel_out_lcd_print(panel, 1, 0, text);
					TRACE("Volume %-3d\n", value);
					break;
			}
		}
	}

	rtems_timer_delete(timer_id);
	panel_exit(panel);

	TRACE("Ending panel_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('T', 'H', 'R', 'U'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create panel_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start panel_task.\n");
		return FALSE;
	}

	return TRUE;
}

static rtems_task dsp_task(rtems_task_argument argument)
{
	int dsp, i;
	rtems_signed32 buffer[2*DSPK_BUFFER_SIZE];

	TRACE("Starting dsp_task...\n");

	dsp = dsp_init(1, dspCode);
	if (!dsp)
		Error("ERROR: cannot access the DSP.\n");

	rtems_task_wake_after(RTEMS_YIELD_PROCESSOR);

	while (TRUE)
	{
		if (!dsp_read_data(dsp, buffer, 2*DSPK_BUFFER_SIZE))
			Error("ERROR: cannot read buffer from DSP.\n");

		for (i=0; i<2*DSPK_BUFFER_SIZE ;i++)
		{
			buffer[i] = (master_volume * ((buffer[i] << 8) >> 16)) >> 8;
		}

		if (!dsp_write_data(dsp, buffer, 2*DSPK_BUFFER_SIZE))
			Error("ERROR: cannot write buffer to DSP.\n");
	}

	dsp_exit(dsp);

	TRACE("Ending dsp_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_boolean create_dsp_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('D', 'S', 'P', ' '),
		10,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create dsp_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, dsp_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start dsp_task.\n");
		return FALSE;
	}

	return TRUE;
}

rtems_task rtems_main(rtems_task_argument ignored)
{
	TRACE("******************************************************************************\n");
	TRACE("ColdFire thru example\n");
	TRACE("******************************************************************************\n");

	create_panel_task();
	create_dsp_task();
	rtems_task_delete(RTEMS_SELF);
}
