/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/HOSTCOMMANDS/main.c                 $
 *     $Date:: 14/05/02 16:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/******************************************************************************
 This example shows the fundamentals of using the DSP host commands to send 
 information from the ColdFire to the DSP.

 It uses the volume potentiometer to adjust the desired volume using a pre-filled 
 table with a variable gain from -90 dB to 0 dB. 
 The Ctrl1 potentiometer is used to control the volume balance between channels L 
 and R. 
 The "Edit" key is used to mute/unmute the audio on the DSP.
 
 Mute and Volume (L&R) information is sent to the DSP by making use of DSP host
 commands. 
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <rtems.h>
#include <chameleon.h>

#include "dsp/dsp_code.h"

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

void Error(char *error)
{
	TRACE(error);
	exit(1);
}

void change_volume(int dsp, float volume, float balance_L, float balance_R)
{
	rtems_signed32 volume_channels[2];
	
	//Compute Volume on Channel L
	volume_channels[0] = float_to_fix_round(volume * balance_L);
	
	//Compute Volume on Channel R	
	volume_channels[1] = float_to_fix_round(volume * balance_R);

	//Send the two volumes to the DSP
	if (!dsp_write_data(dsp, volume_channels, 2))
		Error("ERROR: cannot write to DSP.\n");
	
	// Notify the DSP the Volume change using Host Command 0
	if (!dsp_write_command(dsp, DSPP_VecHostCommandUpdateLevels/2, TRUE))
		Error("ERROR: cannot write command to DSP.\n");
}

static rtems_task panel_task(rtems_task_argument argument)
{
	static float		volume_table[128];
	int			dsp;
	int			panel;
	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	rtems_unsigned32	key_bits;
	char			text[17];
	int			i;
	float			dB;
	float			balance;
	float			master_volume = 0.0f;
	float 			balance_L = 1.0f;
	float 			balance_R = 1.0f;
	BOOL 			mute = FALSE;

	TRACE("Starting panel_task...\n");

	/* Fill volume_table with a variable gain from -60 dB to 0 dB */
	
	for (i=0; i<128 ;i++)
	{
		if (i < 27)
			dB = -90.0f + (float) 40.0f*i/27.0f;
		else
			dB = -50.0f+ (float) 50.0f*(i-27)/100.0f;

		volume_table[i] = powf(10.0, dB/20.0);
	}

	volume_table[0] = 0;

	dsp = dsp_init(1, dspCode);
	if (!dsp)
		Error("ERROR: cannot access the DSP.\n");

	panel = panel_init();
	if (!panel)
		Error("ERROR: cannot access the Panel.\n");

	panel_out_lcd_print(panel, 0, 0, " Host commands  ");
	panel_out_led(panel, PANEL01_LED_CTRL1);

	// Infinite loop
	
	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
			Error("ERROR: unexpected exit waiting new panel event.\n");

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			switch (potentiometer)
			{
				case PANEL01_POT_VOLUME:
					master_volume = volume_table[value];
					change_volume(dsp, master_volume, balance_L, balance_R);
				
					TRACE("Volume %-3d\n", value);
					sprintf(text, "   Volume %-3d   ", value);
					panel_out_lcd_print(panel, 1, 0, text);
					break;
				
				case PANEL01_POT_CTRL1:
					balance = (value - 64.0f)/64.0f;
					
					if (balance == 0.0f)
					{
						balance_L = 1.0f;
						balance_R = 1.0f;
					}
					else if (balance > 0.0f)
					{
						balance_L = 1.0f - balance;
						balance_R = 1.0f;
					} 
					else
					{
						balance_L = 1.0f;
						balance_R = 1.0f + balance;
					}
					
					change_volume(dsp, master_volume, balance_L, balance_R);
					break;
			}
		}
		else if (panel_in_keypad(panel, &key_bits))
		{
			switch(key_bits)
			{
				case PANEL01_KEY_EDIT:
					mute = !mute;
					panel_out_led(panel, mute ? PANEL01_LED_CTRL1|PANEL01_LED_EDIT : PANEL01_LED_CTRL1);
					// Send the Mute/Unmute Command to the DSP using Host
					// Command 1 (interrupt vector at address 0x66 = 2 * 0x33)
					if (!dsp_write_command(dsp, DSPP_VecHostCommandToggleMute/2, TRUE))
						Error("ERROR: cannot write command to DSP.\n");
					break;
			}
		}
	}

	dsp_exit(dsp);
	panel_exit(panel);

	rtems_task_delete(RTEMS_SELF);
	TRACE("Ending panel_task...\n");
}

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('H', 'O', 'S', 'T'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create panel_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start panel_task.\n");
		return FALSE;
	}

	return TRUE;
}


rtems_task rtems_main(rtems_task_argument ignored)
{
	TRACE("******************************************************************************\n");
	TRACE("DSP Host Commands example\n");
	TRACE("******************************************************************************\n");

	create_panel_task();
	rtems_task_delete(RTEMS_SELF);
}
