/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/levelmeter/main.c                   $
 *     $Date:: 14/05/02 17:12                                                  $
 * $Revision:: 4                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/


/*
	LEVEL METER application main file.
*/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <rtems.h>
#include <chameleon.h>

#include "lcd.h"
#include "dsp/dsp_code.h"

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

int		dsp;
int		panel;
rtems_boolean	edit_mode;
rtems_boolean	monitoring_input;

/*
	Auxiliary functions
*/

void Error(char *error)
{
	TRACE(error);
	rtems_task_delete(RTEMS_SELF);
}

void start_dsp()
{
	TRACE("Booting DSP...\n");
	dsp = dsp_init(1, dspCode);
	if (!dsp)
		Error("ERROR: cannot access the DSP.\n");
}


/*******************************************************************************

	PANEL TASK		

	Monitors the user actions on the front panel controls

*******************************************************************************/

void panel_redefine_symbols(int panel)
{
	rtems_unsigned8	i;
	for (i=0; i < sizeof(Level_horz_lcd_symbols)/8; i++ )
		if (!panel_out_lcd_redefine(panel, i+1, Level_horz_lcd_symbols[i]))
			Error("ERROR: cannot redefine LCD symbol.\n");
}

void change_monitor_source()
{
	monitoring_input = !monitoring_input;
	
	if (!dsp_write_flag0(dsp, monitoring_input))
		Error("DSP Write Flag Error.\n");

	if (monitoring_input)
		panel_out_lcd_print(panel, 1, 0, "    <Input >    ");
	else
		panel_out_lcd_print(panel, 1, 0, "    <Output>    ");
}

void change_edit_mode()
{

	/*
		Possible edit states:
			- Showing the level meter bars (edit mode off)
			- Changing the level meter source between input or output (edit mode on)
	*/

	char	text[17];

	edit_mode = !edit_mode;
	
	if (edit_mode)
	{
		/*  Turn the EDIT led on */
		panel_out_led(panel, PANEL01_LED_EDIT);

		/* Display the edit options */
		panel_out_lcd_print(panel, 0, 0, "   Monitoring   ");

		if (monitoring_input)
			panel_out_lcd_print(panel, 1, 0, "    <Input >    ");
		else
			panel_out_lcd_print(panel, 1, 0, "    <Output>    ");
	}
	else
	{
		/* Turn the EDIT led off */
		panel_out_led(panel, 0);

		/* Show the level meter bars */
		sprintf(text,"%c", 4);
		panel_out_lcd_print(panel, 0, 0, text);

		sprintf(text,"%c", 5);
		panel_out_lcd_print(panel, 1, 0, text);
	}
}

/* The PANEL task itself */

static rtems_task panel_task(rtems_task_argument argument)
{
	static rtems_unsigned32	volume_table[128];
	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	rtems_unsigned32	key_bits;
	rtems_unsigned8		encoder;
	rtems_signed8		increment;
	int			i;
	float			dB;

	/* Initialization */

	edit_mode = FALSE;
	monitoring_input = FALSE;

	TRACE("Starting panel_task...\n");

	/* Fill volume_table with a variable gain from -60 dB to 0dB */
	for (i=0; i<128 ;i++)
	{
		if (i < 27)
			dB = -90.0 + (float) 40.0*i/27.0;
		else
			dB = -50.0 + (float) 50.0*(i-27)/100.0;

		volume_table[i] = float_to_fix_round(pow(10.0, dB/20.0));
	}

	volume_table[0] = 0;

	panel = panel_init();
	if (!panel)
		Error("ERROR: cannot access the Panel.\n");

	panel_redefine_symbols(panel);

	/* Endless loop */

	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
			Error("ERROR: unexpected exit waiting new panel event.\n");

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			switch (potentiometer)
			{
				case PANEL01_POT_VOLUME:
					/* Volume change : Writes the new value to the DSP */
					if (!dsp_write_data(dsp, &volume_table[value], 1))
						Error("ERROR: cannot write to DSP.\n");
					break;

				case PANEL01_POT_CTRL1:
					break;
					
				case PANEL01_POT_CTRL2:
					break;

				case PANEL01_POT_CTRL3:
					break;
			}
		}
		else if (panel_in_keypad(panel, &key_bits))
		{
			switch (key_bits)
			{
				case PANEL01_KEY_EDIT:
					/* Toggle edit mode */
					change_edit_mode();
				     	break;	

				case (PANEL01_KEY_VALUE_UP):
					if (edit_mode)
						change_monitor_source();
					break;

				case (PANEL01_KEY_VALUE_DOWN):
					if (edit_mode)
						change_monitor_source();
					break;
			}
		}
		else if (panel_in_encoder(panel, &encoder, &increment))
		{
		}
	}

	/* Should never be here... */
	panel_exit(panel);

	TRACE("Ending panel_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

/* Task specific parameters initialization & task start */

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('P', 'A', 'N', 'L'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot create panel_task.\n");

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot start panel_task.\n");

	return TRUE;
}


/******************************************************************************

 	LEVEL METER TASK		
	Reads data from the DSP and displays it on the LCD

 ******************************************************************************/

static rtems_task level_meter_task(rtems_task_argument argument)
{
	rtems_id		period_id;
	rtems_status_code	status;
	rtems_signed32		level;
	rtems_signed32		level_R;
	rtems_signed32		level_L;
	char			text[17];

	/* Calibration factor */
	const float		norm = log10(2047)/(sizeof(Level_horz_lcd_symbol_table)/16);

	TRACE("Starting level_meter_task...\n");

	/* To read periodically from the DSP */
	status = rtems_rate_monotonic_create(rtems_build_name('P', 'E', 'R', '0'), &period_id);
	if (status != RTEMS_SUCCESSFUL)
		Error("Rate monotonic error");

	/* Print Level Meter bars */
	sprintf(text,"%c",4);
	panel_out_lcd_print (panel, 0, 0, text);

	sprintf(text,"%c",5);
	panel_out_lcd_print (panel, 1, 0, text);

	/* Periodic reads & Display updates (20ms) */
	while (rtems_rate_monotonic_period(period_id, 20) != RTEMS_TIMEOUT)
	{
		if (!dsp_read_data(dsp, &level, 1))
			Error("ERROR: cannot read from DSP.\n");
		else
		{
			/* Extract L value from the readed word */
			level_L = level & 0x00FFF000;
			level_L >>= 12;
			level_L = log10(level_L + 1)/norm;

			/* Extract R value from the readed word */

			level_R = level & 0x00000FFF;
			level_R = log10(level_R + 1)/norm;
			
			if (!edit_mode)
			{
/*
				// Non graphic way to display data:

				sprintf(text,"Level R: %04d",level_R);
				panel_out_lcd_print (panel, 0, 1, text);
				sprintf(text,"Level L: %04d",level_L);
				panel_out_lcd_print (panel, 1, 1, text);
*/
				/* Display the Level Meter data */
				panel_out_lcd_print (panel, 0, 1, Level_horz_lcd_symbol_table[level_R]);
				panel_out_lcd_print (panel, 1, 1, Level_horz_lcd_symbol_table[level_L]);

			}
		}
	}

	/* Should never be here... */
	TRACE("Ending level_meter_task...\n");
	rtems_task_delete(RTEMS_SELF);
}


/* Task specific parameters initialization & task start */

rtems_boolean create_level_meter_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('V', 'U', 'M', 'E'),
		60,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot create level_meter_task.\n");

	status = rtems_task_start(task_id, level_meter_task, 0);
	if (status != RTEMS_SUCCESSFUL)
		Error("ERROR: cannot start level_meter_task.\n");

	return TRUE;
}

/******************************************************************************
	MAIN TASK		
 ******************************************************************************/

rtems_task rtems_main(rtems_task_argument ignored)
{
	TRACE("******************************************************************************\n");
	TRACE("Audio level meter example\n");
	TRACE("******************************************************************************\n");

	start_dsp();
	create_panel_task();
	create_level_meter_task();

	rtems_task_delete(RTEMS_SELF);
}
